@@ -44,9 +44,13 @@
 #include "vcal_prefs.h"
 #include "vcal_manager.h"
 #include "vcal_meeting_gtk.h"
+#include "vcal_interface.h"
 #include "prefs_account.h"
 #include "prefs_common.h"
+#include "claws.h"
+#include "menu.h"
 #include "inputdialog.h"
+#include "inc.h"
 #include "xml.h"
 #include "alertpanel.h"
 #include "log.h"
@@ -54,6 +58,7 @@
 #include "statusbar.h"
 #include "msgcache.h"
 #include "timing.h"
+#include "messageview.h"
 
 #include <gtk/gtk.h>
 #include <dirent.h>
@@ -111,6 +116,7 @@ static void export_cal_cb(FolderView *folderview, guint action, GtkWidget *widge
 static void subscribe_cal_cb(FolderView *folderview, guint action, GtkWidget *widget);
 static void check_subs_cb(FolderView *folderview, guint action, GtkWidget *widget);
 static void unsubscribe_cal_cb(FolderView *folderview, guint action, GtkWidget *widget);
+static void set_view_cb(FolderView *folderview, guint action, GtkWidget *widget);
 static void set_sensitivity(GtkItemFactory *factory, FolderItem *item);
 static void update_subscription(const gchar *uri, gboolean verbose);
 static void rename_cb(FolderView *folderview, guint action, GtkWidget *widget);
@@ -140,6 +146,10 @@ struct _VCalFolderItem
 	GSList *evtlist;
 	gboolean batching;
 	gboolean dirty;
+	day_win *dw;
+	month_win *mw;
+	time_t last_fetch;
+	int use_cal_view;
 };
 
 static char *vcal_popup_labels[] =
@@ -154,8 +164,13 @@ static char *vcal_popup_labels[] =
 	"/---",
 	N_("/U_pdate subscriptions"),
 	"/---",
+	N_("/_List view"),
+	N_("/_Week view"),
+	N_("/_Month view"),
+	"/---",
 	NULL
-};			
+};
+	
 static GtkItemFactoryEntry vcal_popup_entries[] =
 {
 	{NULL, NULL, new_meeting_cb,	0, NULL},
@@ -167,6 +182,10 @@ static GtkItemFactoryEntry vcal_popup_entries[] =
 	{NULL, NULL, rename_cb, 0, NULL},
 	{NULL, NULL, NULL,    	0, "<Separator>"},
 	{NULL, NULL, check_subs_cb, 0, NULL},
+	{NULL, NULL, NULL,  	0, "<Separator>"},
+	{NULL, NULL, set_view_cb, 0, "<RadioItem>"},
+	{NULL, NULL, set_view_cb, 1, "/List view"},
+	{NULL, NULL, set_view_cb, 2, "/List view"},
 	{NULL, NULL, NULL,  	0, "<Separator>"}
 };
 
@@ -199,6 +218,9 @@ static void vcal_fill_popup_menu_labels(void)
 
 	for( i = 0; vcal_popup_labels[i] != NULL; i++ ) {
 		(vcal_popup_entries[i]).path = _(vcal_popup_labels[i]);
+		if (!strcmp2(vcal_popup_entries[i].item_type, "/List view")) {
+			vcal_popup_entries[i].item_type = _(vcal_popup_entries[i].item_type);
+		}
 	}
 }
 
@@ -214,7 +236,8 @@ static void vcal_item_set_xml(Folder *folder, FolderItem *item, XMLTag *tag)
 {
 	GList *cur;
 	folder_item_set_xml(folder, item, tag);
-	
+	gboolean found_cal_view_setting = FALSE;
+
 	for (cur = tag->attr; cur != NULL; cur = g_list_next(cur)) {
 		XMLAttr *attr = (XMLAttr *) cur->data;
 
@@ -224,30 +247,32 @@ static void vcal_item_set_xml(Folder *folder, FolderItem *item, XMLTag *tag)
 				g_free(((VCalFolderItem *)item)->uri);
 			((VCalFolderItem *)item)->uri = g_strdup(attr->value);
 		} 
+		if (!strcmp(attr->name, "use_cal_view")) {
+			found_cal_view_setting = TRUE;
+			((VCalFolderItem *)item)->use_cal_view = atoi(attr->value);
+		} 
 	}
 	if (((VCalFolderItem *)item)->uri == NULL) {
 		/* give a path to inbox */
 		g_free(item->path);
 		item->path = g_strdup(".meetings");
 	}
+	if (!found_cal_view_setting)
+		((VCalFolderItem *)item)->use_cal_view = 1; /*week view */
+	
 }
 
 static XMLTag *vcal_item_get_xml(Folder *folder, FolderItem *item)
 {
-	static gchar *folder_item_stype_str[] = {"normal", "inbox", "outbox",
-						 "draft", "queue", "trash"};
-	static gchar *sort_key_str[] = {"none", "number", "size", "date",
-					"from", "subject", "score", "label",
-					"mark", "unread", "mime", "to", 
-					"locked"};
 	XMLTag *tag;
-	gchar *value;
 
 	tag = folder_item_get_xml(folder, item);
 
 	if (((VCalFolderItem *)item)->uri)
 		xml_tag_add_attr(tag, xml_attr_new("uri", ((VCalFolderItem *)item)->uri));
 
+	xml_tag_add_attr(tag, xml_attr_new_int("use_cal_view", ((VCalFolderItem *)item)->use_cal_view));
+
 	return tag;
 }
 
@@ -268,6 +293,43 @@ static void vcal_get_sort_type(Folder *folder, FolderSortKey *sort_key,
 		*sort_key = SORT_BY_DATE;
 }
 
+static void vcal_item_opened(FolderItem *item)
+{
+	struct tm tmdate;
+	time_t t = time(NULL);
+
+	localtime_r(&t, &tmdate);
+
+	if (!((VCalFolderItem *)(item))->dw 
+	    && ((VCalFolderItem *)(item))->use_cal_view == 1)
+		((VCalFolderItem *)(item))->dw = create_day_win(item, tmdate);		
+	else if (!((VCalFolderItem *)(item))->mw 
+	    && ((VCalFolderItem *)(item))->use_cal_view == 2)
+		((VCalFolderItem *)(item))->mw = create_month_win(item, tmdate);
+	else if (((VCalFolderItem *)(item))->use_cal_view != 0)
+		vcal_folder_refresh_cal(item);
+}
+
+void vcal_folder_refresh_cal(FolderItem *item)
+{
+	if (((VCalFolderItem *)(item))->dw)
+		refresh_day_win(((VCalFolderItem *)(item))->dw);
+	if (((VCalFolderItem *)(item))->mw)
+		refresh_month_win(((VCalFolderItem *)(item))->mw);
+}
+
+static void vcal_item_closed(FolderItem *item)
+{
+	if (((VCalFolderItem *)(item))->dw) {
+		dw_close_window(((VCalFolderItem *)(item))->dw);
+		((VCalFolderItem *)(item))->dw = NULL;
+	}
+	if (((VCalFolderItem *)(item))->mw) {
+		mw_close_window(((VCalFolderItem *)(item))->mw);
+		((VCalFolderItem *)(item))->mw = NULL;
+	}
+}
+
 FolderClass *vcal_folder_get_class()
 {
 	if (vcal_class.idstr == NULL) {
@@ -308,6 +370,9 @@ FolderClass *vcal_folder_get_class()
 		vcal_class.change_flags = vcal_change_flags;
 		vcal_class.subscribe = vcal_subscribe_uri;
 
+		/* FolderView functions */
+		vcal_class.item_opened = vcal_item_opened;
+		vcal_class.item_closed = vcal_item_closed;
 		debug_print("registered class for real\n");
 	}
 
@@ -403,7 +468,6 @@ gboolean manual_update = TRUE;
 static gint feed_fetch(FolderItem *fitem, MsgNumberList ** list, gboolean *old_uids_valid)
 {
 	VCalFolderItem *item = (VCalFolderItem *)fitem;
-	MsgNumberList *msglist = NULL;
 	icalcomponent *evt = NULL;
 	icalcomponent_kind type = ICAL_VEVENT_COMPONENT;
 	gint num = 1;
@@ -566,6 +630,12 @@ add_new:
 		evt = icalcomponent_get_next_component(
 			item->cal, type);
 	}
+	if (today_msg == -1) {
+		IcalFeedData *data = icalfeeddata_new(NULL, EVENT_TODAY_ID);
+		item->numlist = g_slist_prepend(item->numlist, GINT_TO_POINTER(num));
+		today_msg = num++;
+		item->evtlist = g_slist_prepend(item->evtlist, data);
+	}
 	item->numlist = g_slist_reverse(item->numlist);
 	item->evtlist = g_slist_reverse(item->evtlist);
 	
@@ -577,70 +647,61 @@ add_new:
 #define VCAL_FOLDER_ADD_EVENT(event) \
 { \
  \
-	debug_print("should add %d %s? %d %d\n", n_msg, event->uid, status, status == ICAL_PARTSTAT_ACCEPTED); \
-	if (status == ICAL_PARTSTAT_ACCEPTED \
-	||  status == ICAL_PARTSTAT_TENTATIVE) { \
+	*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
+	debug_print("add %d %s\n", n_msg, event->uid); \
+	n_msg++; \
+	days = event_to_today(event, 0); \
+ \
+	if (days == EVENT_PAST && past_msg == -1) { \
 		*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
-		debug_print("add %d %s\n", n_msg, event->uid); \
-		n_msg++; \
-		days = event_to_today(event, 0); \
-	 \
-		if (days == EVENT_PAST && past_msg == -1) { \
-			*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
-			past_msg = n_msg++; \
-			g_hash_table_insert(hash_uids, GINT_TO_POINTER(past_msg), g_strdup(EVENT_PAST_ID)); \
-		} else if (days == EVENT_TODAY && today_msg == -1) { \
-			*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
-			today_msg = n_msg++; \
-			g_hash_table_insert(hash_uids, GINT_TO_POINTER(today_msg), g_strdup(EVENT_TODAY_ID)); \
-		} else if (days == EVENT_TOMORROW && tomorrow_msg == -1) { \
-			*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
-			tomorrow_msg = n_msg++; \
-			g_hash_table_insert(hash_uids, GINT_TO_POINTER(tomorrow_msg), g_strdup(EVENT_TOMORROW_ID)); \
-		} else if (days == EVENT_THISWEEK && thisweek_msg == -1) { \
-			*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
-			thisweek_msg = n_msg++; \
-			g_hash_table_insert(hash_uids, GINT_TO_POINTER(thisweek_msg), g_strdup(EVENT_THISWEEK_ID)); \
-		} else if (days == EVENT_LATER && later_msg == -1) { \
-			*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
-			later_msg = n_msg++; \
-			g_hash_table_insert(hash_uids, GINT_TO_POINTER(later_msg), g_strdup(EVENT_LATER_ID)); \
-		} \
+		past_msg = n_msg++; \
+		g_hash_table_insert(hash_uids, GINT_TO_POINTER(past_msg), g_strdup(EVENT_PAST_ID)); \
+	} else if (days == EVENT_TODAY && today_msg == -1) { \
+		*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
+		today_msg = n_msg++; \
+		g_hash_table_insert(hash_uids, GINT_TO_POINTER(today_msg), g_strdup(EVENT_TODAY_ID)); \
+	} else if (days == EVENT_TOMORROW && tomorrow_msg == -1) { \
+		*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
+		tomorrow_msg = n_msg++; \
+		g_hash_table_insert(hash_uids, GINT_TO_POINTER(tomorrow_msg), g_strdup(EVENT_TOMORROW_ID)); \
+	} else if (days == EVENT_THISWEEK && thisweek_msg == -1) { \
+		*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
+		thisweek_msg = n_msg++; \
+		g_hash_table_insert(hash_uids, GINT_TO_POINTER(thisweek_msg), g_strdup(EVENT_THISWEEK_ID)); \
+	} else if (days == EVENT_LATER && later_msg == -1) { \
+		*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
+		later_msg = n_msg++; \
+		g_hash_table_insert(hash_uids, GINT_TO_POINTER(later_msg), g_strdup(EVENT_LATER_ID)); \
 	} \
 }
 
-static gint vcal_get_num_list(Folder *folder, FolderItem *item,
-				 MsgNumberList ** list, gboolean *old_uids_valid)
+GSList *vcal_get_events_list(FolderItem *item)
 {
 	DIR *dp;
 	struct dirent *d;
-	int n_msg = 1;
-	gint past_msg = -1, today_msg = -1, tomorrow_msg = -1, 
-		thisweek_msg = -1, later_msg = -1;
-	START_TIMING("");
-	g_return_val_if_fail (*list == NULL, 0); /* we expect a NULL list */
+	GSList *events = NULL;
+
+	if (item != item->folder->inbox) {
+		GSList *subs = vcal_folder_get_webcal_events_for_folder(item);
+		GSList *cur = NULL;
+		for (cur = subs; cur; cur = cur->next) {
+			icalcomponent *ical = (icalcomponent *)cur->data;
+			VCalEvent *event = vcal_get_event_from_ical(
+				icalcomponent_as_ical_string(ical), NULL);
+			icalcomponent_free(ical);
+			events = g_slist_prepend(events, event);
+		}
+		g_slist_free(subs);
+		return events;
+	}
 
-	debug_print(" num for %s\n", ((VCalFolderItem *)item)->uri);
-	
-	*old_uids_valid = FALSE;
-	
-	if (((VCalFolderItem *)item)->uri) 
-		return feed_fetch(item, list, old_uids_valid);
-	
 	dp = opendir(vcal_manager_get_event_path());
 	
-	debug_print("get_num_list\n");
 	if (!dp) {
 		FILE_OP_ERROR(vcal_manager_get_event_path(), "opendir");
 		return 0;
 	}
 
-	if (hash_uids != NULL)
-		g_hash_table_destroy(hash_uids);
-		
-	hash_uids = g_hash_table_new_full(g_direct_hash, g_direct_equal,
-					  NULL, g_free);
-	
 	while ((d = readdir(dp)) != NULL) {
 		VCalEvent *event = NULL;
 		if (d->d_name[0] == '.' || strstr(d->d_name, ".bak")
@@ -649,8 +710,6 @@ static gint vcal_get_num_list(Folder *folder, FolderItem *item,
 		||  !strcmp(d->d_name, "multisync")) 
 			continue;
 
-		g_hash_table_insert(hash_uids, GINT_TO_POINTER(n_msg), g_strdup(d->d_name));
-		
 		event = vcal_manager_load_event(d->d_name);
 		if (!event)
 			continue;
@@ -661,11 +720,16 @@ static gint vcal_get_num_list(Folder *folder, FolderItem *item,
 		}
 
 		if (event && event->method != ICAL_METHOD_CANCEL) {
-			EventTime days; \
-			PrefsAccount *account = vcal_manager_get_account_from_event(event); \
-			enum icalparameter_partstat status = \
-				account ? vcal_manager_get_reply_for_attendee(event, account->address): ICAL_PARTSTAT_NEEDSACTION; \
-			VCAL_FOLDER_ADD_EVENT(event);
+			PrefsAccount *account = vcal_manager_get_account_from_event(event);
+			enum icalparameter_partstat status =
+				account ? vcal_manager_get_reply_for_attendee(event, account->address): ICAL_PARTSTAT_NEEDSACTION;
+			if (status == ICAL_PARTSTAT_ACCEPTED
+			||  status == ICAL_PARTSTAT_TENTATIVE) {
+				events = g_slist_prepend(events, event);
+			} else {
+				vcal_manager_free_event(event);
+				continue;
+			}
 			if ((status == ICAL_PARTSTAT_ACCEPTED
 			     || status == ICAL_PARTSTAT_TENTATIVE) 
 			    && event->recur && *(event->recur)) {
@@ -708,22 +772,83 @@ static gint vcal_get_num_list(Folder *folder, FolderItem *item,
 					vcal_manager_copy_attendees(event, nevent);
 					nevent->rec_occurence = TRUE;
 					vcal_manager_save_event(nevent, FALSE);
-					g_hash_table_insert(hash_uids, GINT_TO_POINTER(n_msg), uid);
-					VCAL_FOLDER_ADD_EVENT(nevent);
-					vcal_manager_free_event(nevent);
+					account = vcal_manager_get_account_from_event(event);
+					status =
+						account ? vcal_manager_get_reply_for_attendee(event, account->address): ICAL_PARTSTAT_NEEDSACTION;
+					if (status == ICAL_PARTSTAT_ACCEPTED
+					||  status == ICAL_PARTSTAT_TENTATIVE) {
+						events = g_slist_prepend(events, nevent);
+					} else {
+						vcal_manager_free_event(nevent);
+					}
 					next = icalrecur_iterator_next(ritr);
 					debug_print("next time is %snull\n", icaltime_is_null_time(next)?"":"not ");
 					i++;
 				}
 				icalrecur_iterator_free(ritr);
 			}
+		} else {
+			vcal_manager_free_event(event);
+		}
+	}
+	closedir(dp);
+	return g_slist_reverse(events);
+}
+
+static gint vcal_get_num_list(Folder *folder, FolderItem *item,
+				 MsgNumberList ** list, gboolean *old_uids_valid)
+{
+	int n_msg = 1;
+	gint past_msg = -1, today_msg = -1, tomorrow_msg = -1, 
+		thisweek_msg = -1, later_msg = -1;
+	GSList *events = NULL, *cur;
+	START_TIMING("");
+	g_return_val_if_fail (*list == NULL, 0); /* we expect a NULL list */
+
+	debug_print(" num for %s\n", ((VCalFolderItem *)item)->uri);
+	
+	*old_uids_valid = FALSE;
+	
+	if (((VCalFolderItem *)item)->uri) 
+		return feed_fetch(item, list, old_uids_valid);
+	
+	events = vcal_get_events_list(item);
+	
+	debug_print("get_num_list\n");
+
+	if (hash_uids != NULL)
+		g_hash_table_destroy(hash_uids);
+		
+	hash_uids = g_hash_table_new_full(g_direct_hash, g_direct_equal,
+					  NULL, g_free);
+	
+	for (cur = events; cur; cur = cur->next) {
+		VCalEvent *event = (VCalEvent *)cur->data;
+
+		g_hash_table_insert(hash_uids, GINT_TO_POINTER(n_msg), g_strdup(event->uid));
+		
+		if (event->rec_occurence) {
+			vcal_manager_free_event(event);
+			continue;
+		}
+
+		if (event && event->method != ICAL_METHOD_CANCEL) {
+			EventTime days;
+			VCAL_FOLDER_ADD_EVENT(event);
 		}
 		if (event)
 			vcal_manager_free_event(event);
 
 
 	}
-	closedir(dp);
+
+	if (today_msg == -1) { 
+		*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); 
+		today_msg = n_msg++; 
+		g_hash_table_insert(hash_uids, GINT_TO_POINTER(today_msg), g_strdup(EVENT_TODAY_ID)); 
+	}
+
+	g_slist_free(events);
 	vcal_folder_export(folder);
 
 	vcal_set_mtime(folder, item);
@@ -787,14 +912,10 @@ static MsgInfo *vcal_get_msginfo(Folder * folder,
 static gchar *feed_fetch_item(FolderItem * fitem, gint num)
 {
 	gchar *filename = NULL;
-	gchar *snum = NULL;
-	gchar *uid = NULL;
 	VCalFolderItem *item = (VCalFolderItem *)fitem;
 	GSList *ncur, *ecur;
 	int i = 1;
 	IcalFeedData *data = NULL;
-	icalproperty *prop = NULL;
-	gchar *title = NULL;
 
 	if (!item->numlist) {
 		debug_print("numlist null\n");
@@ -877,11 +998,27 @@ static gchar *vcal_fetch_msg(Folder * folder, FolderItem * item,
 
 static gint vcal_add_msg(Folder *folder, FolderItem *_dest, const gchar *file, MsgFlags *flags)
 {
+	gchar *contents = file_read_to_str(file);
+	if (contents) {
+		vcal_add_event(contents);		
+	}
+	g_free(contents);
 	return 0;
 }
 
+static void vcal_remove_event (Folder *folder, MsgInfo *msginfo);
+
 static gint vcal_remove_msg(Folder *folder, FolderItem *_item, gint num)
 {
+	MsgInfo *msginfo = folder_item_get_msginfo(_item, num);
+
+	if (!msginfo)
+		return 0;
+
+	if (_item == folder->inbox)
+		vcal_remove_event(folder, msginfo);
+
+	procmsg_msginfo_free(msginfo);
 	return 0;
 }
 
@@ -1028,21 +1165,13 @@ void vcal_folder_export(Folder *folder)
 
 static void vcal_remove_event (Folder *folder, MsgInfo *msginfo)
 {
-	MimeInfo *mime = procmime_scan_message(msginfo);
-	gchar *uid = NULL;
+	const gchar *uid = msginfo->msgid;
 	VCalFolderItem *item = (VCalFolderItem *)msginfo->folder;
-	if (mime)
-		mime = procmime_mimeinfo_next(mime);
-	
-	debug_print("next mime info:%s\n", mime?mime->subtype:"(nil)");
-	if (mime && !strcmp(mime->subtype, "calendar")) {
-		uid = vcalviewer_get_uid_from_mimeinfo(mime);
-		if (uid) {
-			gchar *file = vcal_manager_get_event_file(uid);
-			g_free(uid);
-			unlink(file);
-			g_free(file);
-		}
+
+	if (uid) {
+		gchar *file = vcal_manager_get_event_file(uid);
+		unlink(file);
+		g_free(file);
 	}
 	
 	if (!item || !item->batching)
@@ -1057,7 +1186,6 @@ static void vcal_change_flags(Folder *folder, FolderItem *_item, MsgInfo *msginf
 	EventTime date;
 
 	if (newflags & MSG_DELETED) {
-		MainWindow *mainwin = mainwindow_get_mainwindow();
 		/* delete the stuff */
 		msginfo->flags.perm_flags |= MSG_DELETED;
 		vcal_remove_event(folder, msginfo);
@@ -1078,7 +1206,6 @@ static void vcal_change_flags(Folder *folder, FolderItem *_item, MsgInfo *msginf
 		msginfo->flags.perm_flags |= MSG_COLORLABEL_TO_FLAGS(2); /* Red */
 		break;
 	case EVENT_TOMORROW:
-		/* msginfo->flags.perm_flags |= MSG_COLORLABEL_TO_FLAGS(1); /* Orange */
 		break;
 	case EVENT_THISWEEK:
 		break;
@@ -1121,20 +1248,47 @@ void vcal_folder_gtk_done(void)
 	folderview_unregister_popup(&vcal_popup);
 }
 
+static gboolean setting_sensitivity = FALSE;
+
 static void set_sensitivity(GtkItemFactory *factory, FolderItem *fitem)
 {
 	VCalFolderItem *item = (VCalFolderItem *)fitem;
+	GtkWidget *menuitem = NULL;
 
 #define SET_SENS(name, sens) \
 	menu_set_sensitive(factory, name, sens)
 
+	setting_sensitivity = TRUE;
+	switch(item->use_cal_view) {
+	case 0:
+		menuitem = gtk_item_factory_get_item
+			(factory, _("/List view"));
+		break;
+	case 1:
+		menuitem = gtk_item_factory_get_item
+			(factory, _("/Week view"));
+		break;
+	case 2:
+		menuitem = gtk_item_factory_get_item
+			(factory, _("/Month view"));
+		break;
+	default:
+		menuitem = gtk_item_factory_get_item
+			(factory, _("/List view"));
+		break;
+	}
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), TRUE);
+
 	SET_SENS(_("/New meeting..."), item->uri == NULL);
 	SET_SENS(_("/Export calendar..."), TRUE);
 	SET_SENS(_("/Subscribe to webCal..."), item->uri == NULL);
 	SET_SENS(_("/Unsubscribe..."), item->uri != NULL);
 	SET_SENS(_("/Rename..."), folder_item_parent(fitem) != NULL);
 	SET_SENS(_("/Update subscriptions"), TRUE);
-	
+	SET_SENS(_("/List view"), folder_item_parent(fitem) != NULL);
+	SET_SENS(_("/Week view"), folder_item_parent(fitem) != NULL);
+	SET_SENS(_("/Month view"), folder_item_parent(fitem) != NULL);
+	setting_sensitivity = FALSE;
 #undef SET_SENS
 }
 
@@ -1146,50 +1300,13 @@ static void new_meeting_cb(FolderView *folderview, guint action, GtkWidget *widg
 
 GSList * vcal_folder_get_waiting_events(void)
 {
-	DIR *dp;
-	struct dirent *d;
-	GSList *list = NULL;
-
-	dp = opendir(vcal_manager_get_event_path());
-
-	if (!dp) {
-		FILE_OP_ERROR(vcal_manager_get_event_path(), "opendir");
-		return 0;
-	}
-
-	while ((d = readdir(dp)) != NULL) {
-		VCalEvent *event = NULL;
-		if (d->d_name[0] == '.' || strstr(d->d_name, ".bak")
-		||  !strcmp(d->d_name, "internal.ics") 
-		||  !strcmp(d->d_name, "internal.ifb") 
-		||  !strcmp(d->d_name, "multisync")) 
-			continue;
-
-		event = vcal_manager_load_event(d->d_name);
-
-		if (event && event->method != ICAL_METHOD_CANCEL) {
-			PrefsAccount *account = vcal_manager_get_account_from_event(event);
-			enum icalparameter_partstat status = ICAL_PARTSTAT_NEEDSACTION;
-			status = account ? vcal_manager_get_reply_for_attendee(event, account->address): ICAL_PARTSTAT_NEEDSACTION;
-
-			if (status == ICAL_PARTSTAT_ACCEPTED
-			||  status == ICAL_PARTSTAT_TENTATIVE) {
-				list = g_slist_prepend(list, event);
-			} else {
-				vcal_manager_free_event(event);
-			}
-		} else {
-			vcal_manager_free_event(event);
-		}
-	}
-
-	closedir(dp);
-		
-	return g_slist_reverse(list);
+	Folder *folder = folder_find_from_name ("vCalendar", vcal_folder_get_class());
+	return vcal_get_events_list(folder->inbox);
 }
 
 typedef struct _get_webcal_data {
 	GSList *list;
+	FolderItem *item;
 } GetWebcalData;
 
 static gboolean get_webcal_events_func(GNode *node, gpointer user_data)
@@ -1199,6 +1316,9 @@ static gboolean get_webcal_events_func(GNode *node, gpointer user_data)
 	gboolean dummy = FALSE;
 	GSList *list = NULL, *cur = NULL;
 
+	if (data->item && data->item != item)
+		return FALSE;
+
 	feed_fetch(item, &list, &dummy);
 
 	
@@ -1215,6 +1335,22 @@ GSList * vcal_folder_get_webcal_events(void)
 	GetWebcalData *data = g_new0(GetWebcalData, 1);
 	Folder *folder = folder_find_from_name ("vCalendar", vcal_folder_get_class());
 	GSList *list = NULL;
+	data->item = NULL;
+	g_node_traverse(folder->node, G_PRE_ORDER,
+			G_TRAVERSE_ALL, -1, get_webcal_events_func, data);
+
+	list = data->list;
+	g_free(data);
+
+	return g_slist_reverse(list);
+}
+
+GSList * vcal_folder_get_webcal_events_for_folder(FolderItem *item)
+{
+	GetWebcalData *data = g_new0(GetWebcalData, 1);
+	Folder *folder = folder_find_from_name ("vCalendar", vcal_folder_get_class());
+	GSList *list = NULL;
+	data->item = item;
 	g_node_traverse(folder->node, G_PRE_ORDER,
 			G_TRAVERSE_ALL, -1, get_webcal_events_func, data);
 
@@ -1234,10 +1370,8 @@ gchar* get_item_event_list_for_date(FolderItem *item, EventTime date)
 	if (((VCalFolderItem *)item)->uri) {
 		for (cur = ((VCalFolderItem *)item)->evtlist; cur; cur = cur->next) {
 			IcalFeedData *fdata = (IcalFeedData *)cur->data;
-			icalcomponent *event;
 			icalproperty *prop;
 			struct icaltimetype itt;
-			gchar *str = NULL;
 			gchar *summary = NULL;
 			EventTime days;
 			if (!fdata->event)
@@ -1393,7 +1527,7 @@ void *url_read_thread(void *data)
 
 	curl_easy_getinfo(curl_ctx, CURLINFO_RESPONSE_CODE, &response_code);
 	if( response_code >= 400 && response_code < 500 ) {
-		debug_print("VCalendar: got %d\n", response_code);
+		debug_print("VCalendar: got %ld\n", response_code);
 		switch(response_code) {
 			case 401: 
 				td->error = g_strdup(_("401 (Authorisation required)"));
@@ -1405,7 +1539,7 @@ void *url_read_thread(void *data)
 				td->error = g_strdup(_("404 (Not found)"));
 				break;
 			default:
-				td->error = g_strdup_printf(_("Error %d"), response_code);
+				td->error = g_strdup_printf(_("Error %ld"), response_code);
 				break;
 		}
 	}
@@ -1430,7 +1564,6 @@ gchar *vcal_curl_read(const char *url, gboolean verbose,
 #endif
 	gchar *msg;
 	void *res;
-	time_t start_time;
 	gboolean killed;
 	gchar *error = NULL;
 	result = NULL;
@@ -1485,7 +1618,7 @@ gboolean vcal_curl_put(gchar *url, FILE *fp, gint filesize, const gchar *user, c
 {
 	gboolean res = TRUE;
 	CURL *curl_ctx = curl_easy_init();
-	int response_code = 0;
+	long response_code = 0;
 	gchar *t_url = url;
 	gchar *userpwd = NULL;
 
@@ -1549,7 +1682,6 @@ static gboolean folder_item_find_func(GNode *node, gpointer data)
 
 static FolderItem *get_folder_item_for_uri(const gchar *uri)
 {
-	FolderItem *tmp = NULL;
 	Folder *root = folder_find_from_name ("vCalendar", vcal_folder_get_class());
 	gpointer d[2];
 	
@@ -1673,10 +1805,13 @@ static void update_subscription_finish(const gchar *uri, gchar *feed, gboolean v
 	((VCalFolderItem *)item)->cal = cal;
 	
 	main_window_cursor_normal(mainwindow_get_mainwindow());
+	((VCalFolderItem *)item)->last_fetch = time(NULL);
 }
 
 static void update_subscription(const gchar *uri, gboolean verbose)
 {
+	FolderItem *item = get_folder_item_for_uri(uri);
+	
 	if (prefs_common.work_offline) {
 		if (!verbose || 
 		!inc_offline_should_override(TRUE,
@@ -1684,7 +1819,11 @@ static void update_subscription(const gchar *uri, gboolean verbose)
 		     "to update the Webcal feed.")))
 			return;
 	}
-	
+	if (item) {
+		if (time(NULL) - ((VCalFolderItem *)(item))->last_fetch < 60 && 
+		    ((VCalFolderItem *)(item))->cal)
+			return;
+	}
 	main_window_cursor_wait(mainwindow_get_mainwindow());
 	vcal_curl_read(uri, verbose, update_subscription_finish);
 }
@@ -1757,6 +1896,8 @@ static void unsubscribe_cal_cb(FolderView *folderview, guint action, GtkWidget *
 	Xstrdup_a(old_path, item->path, return);
 	old_id = folder_item_get_identifier(item);
 
+	vcal_item_closed(item);
+
 	if (folderview->opened == folderview->selected ||
 	    gtk_ctree_is_ancestor(ctree,
 				  folderview->selected,
@@ -1839,3 +1980,369 @@ static void rename_cb(FolderView *folderview, guint action,
 	folder_item_prefs_save_config_recursive(item);
 	folder_write_list();
 }
+
+static void set_view_cb(FolderView *folderview, guint action, GtkWidget *widget)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+	FolderItem *item = NULL, *oitem = NULL;
+
+	if (!folderview->selected) return;
+	if (setting_sensitivity) return;
+
+	oitem = gtk_ctree_node_get_row_data(ctree, folderview->opened);
+	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
+
+	if (((VCalFolderItem *)(item))->use_cal_view == action)
+		return;
+	debug_print("set view %d\n", action);
+	if (oitem && item == oitem && oitem->folder->klass == vcal_folder_get_class())
+		oitem->folder->klass->item_closed(oitem);
+	((VCalFolderItem *)(item))->use_cal_view = action;
+	if (((VCalFolderItem *)(item))->use_cal_view) {
+		if (oitem && item == oitem && oitem->folder->klass == vcal_folder_get_class())
+			oitem->folder->klass->item_opened(oitem);
+	}
+}
+
+gchar *vcal_get_event_as_ical_str(VCalEvent *event)
+{
+	gchar *ical;
+	icalcomponent *calendar = icalcomponent_vanew(
+            ICAL_VCALENDAR_COMPONENT,
+	    icalproperty_new_version("2.0"),
+            icalproperty_new_prodid(
+                 "-//Claws Mail//NONSGML Claws Mail Calendar//EN"),
+	    icalproperty_new_calscale("GREGORIAN"),
+            0);
+	vcal_manager_event_dump(event, FALSE, FALSE, calendar, FALSE);
+	ical = g_strdup(icalcomponent_as_ical_string(calendar));
+	icalcomponent_free(calendar);
+	
+	return ical;
+}
+
+static gchar *get_name_from_property(icalproperty *p)
+{
+	gchar *tmp = NULL;
+	
+	if (p && icalproperty_get_parameter_as_string(p, "CN") != NULL)
+		tmp = g_strdup(icalproperty_get_parameter_as_string(p, "CN"));
+
+	return tmp;
+}
+
+static gchar *get_email_from_property(icalproperty *p)
+{
+	gchar *tmp = NULL;
+	gchar *email = NULL;
+	
+	if (p)
+		tmp = g_strdup(icalproperty_get_organizer(p));
+
+	if (!tmp) 
+		return NULL;
+
+	if (!strncasecmp(tmp, "MAILTO:", strlen("MAILTO:")))
+		email = g_strdup(tmp+strlen("MAILTO:"));
+	else
+		email = g_strdup(tmp);
+	g_free(tmp);
+	
+	return email;
+}
+
+#define GET_PROP(comp,prop,kind) {						\
+	prop = NULL;								\
+	if (!(prop = icalcomponent_get_first_property(comp, kind))) {		\
+		prop = inner							\
+			? icalcomponent_get_first_property(inner, kind)		\
+			: NULL;							\
+	}									\
+}
+
+#define GET_PROP_LIST(comp,list,kind) {						\
+	list = NULL;								\
+	if (!(prop = icalcomponent_get_first_property(comp, kind))) {		\
+		prop = inner							\
+			? icalcomponent_get_first_property(inner, kind)		\
+			: NULL;							\
+		if (prop) do {							\
+			list = g_slist_prepend(list, prop);			\
+		} while ((prop = icalcomponent_get_next_property(inner, kind)));\
+	}else do {								\
+		list = g_slist_prepend(list, prop);				\
+	} while ((prop = icalcomponent_get_next_property(comp, kind)));		\
+}
+
+#define TO_UTF8(string) {							\
+	if (string && !g_utf8_validate(string, -1, NULL)) {			\
+		gchar *tmp = conv_codeset_strdup(string, 			\
+				charset ? charset:conv_get_locale_charset_str(),\
+				CS_UTF_8);					\
+		g_free(string);							\
+		string = tmp;							\
+	}									\
+}
+
+VCalEvent *vcal_get_event_from_ical(const gchar *ical, const gchar *charset)
+{
+	VCalEvent *event = NULL;
+	gchar *int_ical = g_strdup(ical);
+	icalcomponent *comp = icalcomponent_new_from_string(int_ical);
+	icalcomponent *inner = NULL;
+	icalproperty *prop = NULL;
+	GSList *list = NULL, *cur = NULL;
+	gchar *uid = NULL;
+	gchar *summary = NULL;
+	gchar *dtstart = NULL;
+	gchar *dtend = NULL;
+	gchar *org_email = NULL, *org_name = NULL;
+	gchar *description = NULL;
+	gchar *url = NULL;
+	gchar *tzid = NULL;
+	gchar *recur = NULL;
+	int sequence = 0;
+	enum icalproperty_method method = ICAL_METHOD_REQUEST;
+	enum icalproperty_kind type = ICAL_VEVENT_COMPONENT;
+	GSList *attendees = NULL;
+	
+	if (comp == NULL) {
+		g_free(int_ical);
+		return NULL;
+	}
+
+	if ((inner = icalcomponent_get_inner(comp)) != NULL)
+	    type = icalcomponent_isa(inner);
+
+	GET_PROP(comp, prop, ICAL_UID_PROPERTY);
+	if (prop) {
+		uid = g_strdup(icalproperty_get_uid(prop));
+		TO_UTF8(uid);
+		icalproperty_free(prop);
+	}
+	GET_PROP(comp, prop, ICAL_SUMMARY_PROPERTY);
+	if (prop) {
+		summary = g_strdup(icalproperty_get_summary(prop));
+		TO_UTF8(summary);
+		icalproperty_free(prop);
+	}
+	GET_PROP(comp, prop, ICAL_DTSTART_PROPERTY);
+	if (prop) {
+		dtstart = g_strdup(icaltime_as_ical_string(icalproperty_get_dtstart(prop)));
+		TO_UTF8(dtstart);
+		icalproperty_free(prop);
+	}
+	GET_PROP(comp, prop, ICAL_DTEND_PROPERTY);
+	if (prop) {
+		dtend = g_strdup(icaltime_as_ical_string(icalproperty_get_dtend(prop)));
+		TO_UTF8(dtend);
+		icalproperty_free(prop);
+	} else {
+		GET_PROP(comp, prop, ICAL_DURATION_PROPERTY);
+		if (prop) {
+			struct icaldurationtype duration = icalproperty_get_duration(prop);
+			struct icaltimetype itt;
+			icalproperty_free(prop);
+			GET_PROP(comp, prop, ICAL_DTSTART_PROPERTY);
+			itt = icalproperty_get_dtstart(prop);
+			if (prop) {
+				icalproperty_free(prop);
+				dtend = g_strdup(icaltime_as_ical_string(icaltime_add(itt,duration)));
+				TO_UTF8(dtend);
+			}
+		}
+	}
+	GET_PROP(comp, prop, ICAL_SEQUENCE_PROPERTY);
+	if (prop) {
+		sequence = icalproperty_get_sequence(prop);
+		icalproperty_free(prop);
+	}
+	GET_PROP(comp, prop, ICAL_METHOD_PROPERTY);
+	if (prop) {
+		method = icalproperty_get_method(prop);
+		icalproperty_free(prop);
+	}
+	GET_PROP(comp, prop, ICAL_ORGANIZER_PROPERTY);
+	if (prop) {
+		org_email = get_email_from_property(prop);
+		TO_UTF8(org_email);
+		org_name = get_name_from_property(prop);
+		TO_UTF8(org_name);
+		icalproperty_free(prop);
+	}
+	GET_PROP(comp, prop, ICAL_DESCRIPTION_PROPERTY);
+	if (prop) {
+		description = g_strdup(icalproperty_get_description(prop));
+		TO_UTF8(description);
+		icalproperty_free(prop);
+	}
+	GET_PROP(comp, prop, ICAL_URL_PROPERTY);
+	if (prop) {
+		url = g_strdup(icalproperty_get_url(prop));
+		TO_UTF8(url);
+		icalproperty_free(prop);
+	}
+	GET_PROP(comp, prop, ICAL_TZID_PROPERTY);
+	if (prop) {
+		tzid = g_strdup(icalproperty_get_tzid(prop));
+		TO_UTF8(tzid);
+		icalproperty_free(prop);
+	}
+	GET_PROP(comp, prop, ICAL_RRULE_PROPERTY);
+	if (prop) {
+		struct icalrecurrencetype rrule = icalproperty_get_rrule(prop);
+		recur = g_strdup(icalrecurrencetype_as_string(&rrule));
+		TO_UTF8(recur);
+		icalproperty_free(prop);
+	}
+	GET_PROP_LIST(comp, list, ICAL_ATTENDEE_PROPERTY);
+	for (cur = list; cur; cur = cur->next) {
+		enum icalparameter_partstat partstat = 0;
+		enum icalparameter_cutype cutype = 0;
+		icalparameter *param = NULL;
+		gchar *email = NULL;
+		gchar *name = NULL;
+		Answer *answer = NULL;
+
+		prop = (icalproperty *)(cur->data);
+
+		email = get_email_from_property(prop);
+		TO_UTF8(email);
+		name = get_name_from_property(prop);
+		TO_UTF8(name);
+
+		param = icalproperty_get_first_parameter(prop, ICAL_PARTSTAT_PARAMETER);
+		if (param)
+			partstat = icalparameter_get_partstat(param);
+
+		param = icalproperty_get_first_parameter(prop, ICAL_CUTYPE_PARAMETER);
+		if (param)
+			cutype= icalparameter_get_cutype(param);
+		
+		if (!partstat)
+			partstat = ICAL_PARTSTAT_NEEDSACTION;
+		if (!cutype)
+			cutype = ICAL_CUTYPE_INDIVIDUAL;
+		answer = answer_new(email, name, partstat, cutype);
+		attendees = g_slist_prepend(attendees, answer);
+		g_free(email);
+		g_free(name);
+		icalproperty_free(prop);
+	}
+	g_slist_free(list);
+	
+	event = vcal_manager_new_event	(uid, org_email, org_name,
+					 summary, description,
+					 dtstart, dtend, recur,
+					 tzid, url,
+					 method, sequence, type);
+	event->answers = attendees;
+	
+	g_free(int_ical);
+	return event;
+}
+
+gboolean vcal_event_exists(const gchar *id)
+{
+	MsgInfo *info = NULL;
+	Folder *folder = folder_find_from_name ("vCalendar", vcal_folder_get_class());
+	if (!folder)
+		return FALSE;
+
+	info = folder_item_get_msginfo_by_msgid(folder->inbox, id);
+	if (info != NULL) {
+		procmsg_msginfo_free(info);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+void vcal_foreach_event(gboolean (*cb_func)(const gchar *vevent))
+{
+	GSList *list = vcal_folder_get_waiting_events();
+	GSList *cur = NULL;
+	if (!cb_func)
+		return;
+	debug_print("calling cb_func...\n");
+	for (cur = list; cur; cur = cur->next) {
+		VCalEvent *event = (VCalEvent *)cur->data;
+		gchar *tmp = vcal_get_event_as_ical_str(event);
+		if (tmp) {
+			debug_print(" ...for event %s\n", event->uid);
+			cb_func(tmp);
+		}
+		vcal_manager_free_event(event);
+		g_free(tmp);
+	}
+}
+
+/* please call vcalendar_refresh_folder_contents() after one or more 
+ * calls to this function */
+gboolean vcal_delete_event(const gchar *id)
+{
+	MsgInfo *info = NULL;
+	Folder *folder = folder_find_from_name ("vCalendar", vcal_folder_get_class());
+	if (!folder)
+		return FALSE;
+
+	info = folder_item_get_msginfo_by_msgid(folder->inbox, id);
+	if (info != NULL) {
+		debug_print("removing event %s\n", id);
+		vcal_remove_event(folder, info);
+		procmsg_msginfo_free(info);
+		folder_item_scan(folder->inbox);
+		return TRUE;
+	}
+	debug_print("not removing unexisting event %s\n", id);
+	return FALSE;
+}
+
+/* please call vcalendar_refresh_folder_contents() after one or more 
+ * calls to this function */
+gchar* vcal_add_event(const gchar *vevent)
+{
+	VCalEvent *event = vcal_get_event_from_ical(vevent, NULL);
+	gchar *retVal = NULL;
+	Folder *folder = folder_find_from_name ("vCalendar", vcal_folder_get_class());
+	if (!folder)
+		return NULL;
+
+	if (event) {
+		if (vcal_event_exists(event->uid)) {
+			debug_print("event %s already exists\n", event->uid);
+			vcal_manager_free_event(event);
+			return retVal;
+		}
+		debug_print("adding event %s\n", event->uid);
+		if (!account_find_from_address(event->organizer, FALSE) &&
+		    !vcal_manager_get_account_from_event(event)) {
+			PrefsAccount *account = account_get_default();
+			vcal_manager_update_answer(event, account->address, 
+					account->name,
+					ICAL_PARTSTAT_ACCEPTED, 
+					ICAL_CUTYPE_INDIVIDUAL);
+			debug_print("can't find our accounts in event, adding default\n");
+		}
+		vcal_manager_save_event(event, TRUE);
+		folder_item_scan(folder->inbox);
+		retVal = vcal_get_event_as_ical_str(event);
+		vcal_manager_free_event(event);
+	}
+
+	return retVal;
+}
+
+/* please call vcalendar_refresh_folder_contents() after one or more 
+ * calls to this function */
+gchar* vcal_update_event(const gchar *vevent)
+{
+	VCalEvent *event = vcal_get_event_from_ical(vevent, NULL);
+	gboolean r = FALSE;
+	if (event) {
+		r = vcal_delete_event(event->uid);
+		vcal_manager_free_event(event);
+		if (r)
+			return vcal_add_event(vevent);
+	}
+	return NULL;
+}
