@@ -31,12 +31,13 @@
 #include <curl/curl.h>
 #include <curl/curlver.h>
 
-#include "utils.h"
 #include "vcalendar.h"
 #include "vcal_folder.h"
 #include "vcal_manager.h"
 #include "vcal_meeting_gtk.h"
 #include "vcal_prefs.h"
+#include "common-views.h"
+#include "mainwindow.h"
 #include "prefs_account.h"
 #include "account.h"
 #include "filesel.h"
@@ -44,6 +45,7 @@
 #include "addr_compl.h"
 #include "gtkutils.h"
 #include "log.h"
+#include "utils.h"
 
 struct _VCalMeeting
 {
@@ -213,26 +215,6 @@ static gboolean avail_btn_can_be_sensitive(void)
 		return TRUE;
 }
 
-static gchar *get_month_name(gint i)
-{
-	struct tm *lt;
-	time_t t;
-	gchar day[4], mon[4];
-	gint dd, hh, mm, ss, yyyy;
-	struct tm buft;
-	
-	tzset();
-	
-	t = time(NULL);
-	lt = localtime_r(&t, &buft);
-	lt->tm_mon = i;
-	
-	sscanf(asctime(lt), "%3s %3s %d %d:%d:%d %d\n",
-	       day, mon, &dd, &hh, &mm, &ss, &yyyy);
-	
-	return g_strdup(mon);
-}
-
 static gint get_dtdate(const gchar *str, gint field)
 {
 	time_t t = icaltime_as_timet((icaltime_from_string(str)));
@@ -959,8 +941,7 @@ static gboolean check_attendees_availability(VCalMeeting *meet, gboolean tell_if
 	gchar *tmp = NULL;
 	gchar *real_url = NULL;
 	gint num_format = 0;
-	gint change_user = -1, change_dom = -1;
-	gchar *problems = NULL;
+	gchar *change_user = NULL, *change_dom = NULL;
 	gchar *dtstart = NULL;
 	gchar *dtend = NULL;
 	gboolean find_avail = FALSE;
@@ -992,11 +973,11 @@ static gboolean check_attendees_availability(VCalMeeting *meet, gboolean tell_if
 
 		tmp = NULL;
 		if (strstr(real_url, "%u") != NULL) {
-			change_user = (gint)strstr(real_url, "%u");
+			change_user = strstr(real_url, "%u");
 			*(strstr(real_url, "%u")+1) = 's';
 		} 
 		if (strstr(real_url, "%d") != NULL) {
-			change_dom = (gint)strstr(real_url, "%d");
+			change_dom = strstr(real_url, "%d");
 			*(strstr(real_url, "%d")+1) = 's';
 		} 
 		debug_print("url format %s\n", real_url);
@@ -1056,14 +1037,14 @@ static gboolean check_attendees_availability(VCalMeeting *meet, gboolean tell_if
 				domain = g_strdup("");
 			}
 			g_free(remail);
-			if (change_user > 0 && change_dom > 0) {
+			if (change_user && change_dom) {
 				if (change_user < change_dom)
 					tmp = g_strdup_printf(real_url, user, domain);
 				else
 					tmp = g_strdup_printf(real_url, domain, user);
-			} else if (change_user > 0) {
+			} else if (change_user) {
 				tmp = g_strdup_printf(real_url, user);
-			} else if (change_dom > 0) {
+			} else if (change_dom) {
 				tmp = g_strdup_printf(real_url, domain);
 			} else {
 				tmp = g_strdup(real_url);
@@ -1189,12 +1170,21 @@ static gboolean send_meeting_cb(GtkButton *widget, gpointer data)
 	PrefsAccount *account = NULL;
 	gboolean res = FALSE;
 	gboolean found_att = FALSE;
+	Folder *folder = folder_find_from_name ("vCalendar", vcal_folder_get_class());
+	gboolean redisp = FALSE;
 
 	if (meet->uid == NULL && meet->visible && 
 	    !check_attendees_availability(meet, FALSE, TRUE)) {
 		return FALSE;
 	}
 
+	if (folder) {
+		MainWindow *mainwin = mainwindow_get_mainwindow();
+		if (mainwin->summaryview->folder_item == folder->inbox) {
+			redisp = TRUE;
+			summary_show(mainwin->summaryview, NULL);
+		}
+	}
 	gtk_widget_set_sensitive(meet->save_btn, FALSE);
 	gtk_widget_set_sensitive(meet->avail_btn, FALSE);
 	if (meet->window->window)
@@ -1202,7 +1192,7 @@ static gboolean send_meeting_cb(GtkButton *widget, gpointer data)
 
 	organizer	= get_organizer(meet);
 	organizer_name	= get_organizer_name(meet);
-	account		= account_find_from_address(organizer);
+	account		= account_find_from_address(organizer, FALSE);
 
 	if (account && account->set_domain && account->domain) {
 		g_snprintf(buf, sizeof(buf), "%s", account->domain); 
@@ -1297,13 +1287,15 @@ static gboolean send_meeting_cb(GtkButton *widget, gpointer data)
 	} else {
 		alertpanel_error(_("Could not send the meeting invitation.\n"
 				   "Check the recipients."));
-		return res;
 	}
-	if (!found_att) {
-		Folder *folder = folder_find_from_name ("vCalendar", vcal_folder_get_class());
-		if (folder)
-			folder_item_scan(folder->inbox);
-		vcalviewer_reload();
+
+	if (folder)
+		folder_item_scan(folder->inbox);
+
+	if (folder && redisp) {
+		MainWindow *mainwin = mainwindow_get_mainwindow();
+		summary_show(mainwin->summaryview, folder->inbox);
+		vcal_folder_refresh_cal(folder->inbox);
 	}
 
 	return res;
@@ -1326,13 +1318,17 @@ static VCalMeeting *vcal_meeting_create_real(VCalEvent *event, gboolean visible)
 {
 	VCalMeeting *meet = g_new0(VCalMeeting, 1);
 	GtkTextBuffer *buffer = NULL;
-	GtkWidget *date_hbox, *date_vbox, *save_hbox, *label, *vbox, *hbox;
+	GtkWidget *date_hbox, *date_vbox, *save_hbox, *label, *hbox;
 	gchar *s = NULL;
 	GtkObject *start_h_adj, *start_m_adj, *end_h_adj, *end_m_adj;
 	int i = 0, num = 0;
 	GtkWidget *scrolledwin;
 	GList *times = NULL;
 	GList *accounts;
+#ifdef MAEMO
+	GtkWidget *notebook;
+	GtkWidget *maemo_vbox0;
+#endif
 	
 	if (!watch_cursor)
 		watch_cursor = gdk_cursor_new(GDK_WATCH);
@@ -1648,7 +1644,6 @@ static VCalMeeting *vcal_meeting_create_real(VCalEvent *event, gboolean visible)
 	TABLE_ADD_LINE(_("Attendees:"), meet->attendees_vbox, FALSE, TRUE);
 	TABLE_ADD_LINE("", date_hbox, TRUE, FALSE);
 	
-	GtkWidget *notebook;
 	notebook = gtk_notebook_new ();
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
@@ -1660,7 +1655,7 @@ static VCalMeeting *vcal_meeting_create_real(VCalEvent *event, gboolean visible)
 			gtk_label_new_with_mnemonic(_("Time:")));
 	gtk_widget_show (notebook);
 	
-	GtkWidget *maemo_vbox0 = gtk_vbox_new(FALSE, 3);
+	maemo_vbox0 = gtk_vbox_new(FALSE, 3);
 	gtk_box_pack_start(GTK_BOX(maemo_vbox0), notebook, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(maemo_vbox0), save_hbox, FALSE, FALSE, 0);
 	
@@ -1690,6 +1685,46 @@ VCalMeeting *vcal_meeting_create(VCalEvent *event)
 	return vcal_meeting_create_real(event, TRUE);
 }
 
+VCalMeeting *vcal_meeting_create_with_start(VCalEvent *event, struct tm *sdate)
+{
+	VCalMeeting *meet = vcal_meeting_create(event);
+	int num = -1;
+	gtk_calendar_select_day(GTK_CALENDAR(meet->start_c),
+				sdate->tm_mday);
+	gtk_calendar_select_day(GTK_CALENDAR(meet->end_c),
+				sdate->tm_mday);
+
+	gtk_calendar_select_month(GTK_CALENDAR(meet->start_c),
+				sdate->tm_mon, sdate->tm_year+1900);
+	gtk_calendar_select_month(GTK_CALENDAR(meet->end_c),
+				sdate->tm_mon, sdate->tm_year+1900);
+
+	if (sdate->tm_hour != 0) {
+		num = get_list_item_num(sdate->tm_hour, 0);
+		if (num > -1)
+			gtk_list_select_item(GTK_LIST(GTK_COMBO(meet->start_time)->list), num);
+		if (sdate->tm_hour < 23) {
+			num = get_list_item_num(sdate->tm_hour+1, 0);
+			if (num > -1)
+				gtk_list_select_item(GTK_LIST(GTK_COMBO(meet->end_time)->list), num);
+		} else {
+			struct tm tm_tomorrow;
+			tm_tomorrow.tm_mday = sdate->tm_mday;
+			tm_tomorrow.tm_mon = sdate->tm_mon;
+			tm_tomorrow.tm_wday = sdate->tm_wday;
+			tm_tomorrow.tm_year = sdate->tm_year+1900;
+			tm_tomorrow.tm_hour = sdate->tm_hour;
+			orage_move_day(&tm_tomorrow, +1);
+			gtk_calendar_select_day(GTK_CALENDAR(meet->end_c),
+						tm_tomorrow.tm_mday);
+			gtk_calendar_select_month(GTK_CALENDAR(meet->end_c),
+						tm_tomorrow.tm_mon, tm_tomorrow.tm_year);
+			gtk_list_select_item(GTK_LIST(GTK_COMBO(meet->end_time)->list), 0);
+		}
+	}
+	return meet;
+}
+
 VCalMeeting *vcal_meeting_create_hidden(VCalEvent *event)
 {
 	return vcal_meeting_create_real(event, FALSE);
@@ -1700,21 +1735,9 @@ gboolean vcal_meeting_send(VCalMeeting *meet)
 	return send_meeting_cb(NULL, meet);
 }
 
-static gboolean g_slist_find_string(GSList *list, const gchar * str)
-{
-	GSList *cur = list;
-	while (cur) {
-		if (!strcmp((gchar *)cur->data, str))
-			return TRUE;
-		cur = cur->next;
-	}
-	return FALSE;
-}
-
 gint vcal_meeting_alert_check(gpointer data)
 {
 	GSList *events = NULL, *cur = NULL;
-	static GSList *alert_done = NULL;
 
 	if (!vcalprefs.alert_enable)
 		return TRUE;
@@ -1872,10 +1895,12 @@ void multisync_export(void)
 	if (fp) {
 		for (cur = files; cur; cur = cur->next) {
 			file = (char *)cur->data;
-			fprintf(fp, "1 1 %s\n", file);
+			if (fprintf(fp, "1 1 %s\n", file) < 0)
+				perror(file);
 			g_free(file);
 		}
-		fclose(fp);
+		if (fclose(fp) == EOF)
+			perror(file);
 	} else {
 		perror(file);
 	}
@@ -2012,113 +2037,6 @@ putfile:
 	return res;
 }
 
-#if 0
-typedef struct _BusySpot BusySpot;
-struct _BusySpot {
-	time_t start;
-	time_t end;
-};
-
-static GSList *vcal_g_slist_merge(GSList *list, BusySpot *spot, gboolean append_if_not_merged)
-{
-	BusySpot *newspot = g_new0(BusySpot, 1);
-	newspot->start = spot->start;
-	newspot->end = spot->end;
-	gchar a[100], b[100], c[100], d[100];
-
-	if (list == NULL) {
-		debug_print("%p (%s-%s) fits nowhere because list empty, %smerging\n",
-			spot, ctime_r(&spot->start, a), ctime_r(&spot->end, b),
-			append_if_not_merged?"":"not ");
-		if (append_if_not_merged)
-			return g_slist_append(NULL, newspot);
-		else 
-			return NULL;
-	} else {
-		GSList *cur = list;
-		for (; cur; cur = cur->next) {
-			BusySpot *curspot = (BusySpot *)cur->data;
-			if (curspot == spot) {
-				/* same one ! don't merge. */
-				debug_print("not merging %p (%s-%s) with itself\n", spot, ctime_r(&spot->start, a),
-					ctime_r(&spot->end, b));
-				continue;
-			} else if (curspot->start <= newspot->start && curspot->end >= newspot->end) {
-				/* fits in, remove the new one */
-				debug_print("%p:\n%s%s fits in %p:\n%s%s\n\n",
-					spot, ctime_r(&spot->start, a), ctime_r(&spot->end, b),
-					curspot, ctime_r(&curspot->start, c), ctime_r(&curspot->end, d));
-				g_free(newspot);
-				return list;
-			} else if (newspot->start < curspot->start && newspot->end > curspot->end) {
-				/* the new one extends the existing one */
-				debug_print("%p:\n%s%s extends %p:\n%s%s\n\n",
-					spot, ctime_r(&spot->start, a), ctime_r(&spot->end, b),
-					curspot, ctime_r(&curspot->start, c), ctime_r(&curspot->end, d));
-				curspot->start = newspot->start;
-				curspot->end = newspot->end;
-				g_free(newspot);
-				return list;
-			} else if (newspot->start < curspot->start && newspot->end >= curspot->start) {
-				/* the new one starts before, update the existing */
-				debug_print("%p:\n%s%s starts before %p:\n%s%s\n\n",
-					spot, ctime_r(&spot->start, a), ctime_r(&spot->end, b),
-					curspot, ctime_r(&curspot->start, c), ctime_r(&curspot->end, d));
-				curspot->start = newspot->start;
-				g_free(newspot);
-				return list;
-			} else if (newspot->end > curspot->end && newspot->start <= curspot->end) {
-				/* the new one ends after, update the existing */
-				debug_print("%p:\n%s%s ends after %p:\n%s%s\n\n",
-					spot, ctime_r(&spot->start, a), ctime_r(&spot->end, b),
-					curspot, ctime_r(&curspot->start, c), ctime_r(&curspot->end, d));
-				curspot->end = newspot->end;
-				g_free(newspot);
-				return list;
-			} else {
-				/* they don't overlap. continue searching. */
-			}
-		}
-		debug_print("%p (%s-%s) fits nowhere, %sadding\n",
-			spot, ctime_r(&spot->start, a), ctime_r(&spot->end, b),
-			append_if_not_merged?"":"not ");
-
-		/* if we reach that, we found nowhere to overlap, so insert */
-		if (append_if_not_merged)
-			return g_slist_append(list, newspot);
-		else
-			return list;
-	}
-}
-
-static GSList *vcal_g_slist_merge_all(GSList *list)
-{
-	int length = 0;
-	
-merge_again:
-	length = g_slist_length(list);
-	
-	if (length <= 1) {
-		/* unmergeable */
-		return list;
-	} else {
-		GSList *cur = list;
-		GSList *new_list = NULL;
-		for (cur = list; cur; cur = cur->next) {
-			new_list = vcal_g_slist_merge(new_list, (BusySpot *)cur->data, TRUE);
-			g_free(cur->data);
-		}
-		g_slist_free(list);
-		list = new_list;
-		if (g_slist_length(new_list) < length) {
-			/* we did merge stuff */
-			goto merge_again;
-		}
-		return list;
-	}
-}
-#endif
-
 gboolean vcal_meeting_export_freebusy(const gchar *path, const gchar *user,
 				const gchar *pass)
 {
@@ -2133,7 +2051,6 @@ gboolean vcal_meeting_export_freebusy(const gchar *path, const gchar *user,
 	time_t whole_start = time(NULL);
 	time_t whole_end = whole_start + (60*60*24*365);
 	gboolean res = TRUE;
-	GSList *busy_spots = NULL;
 	struct icaltimetype itt_start, itt_end;
 	long filesize = 0;
 	
@@ -2169,22 +2086,6 @@ gboolean vcal_meeting_export_freebusy(const gchar *path, const gchar *user,
 
 	icalcomponent_add_component(calendar, timezone);
 
-#if 0
-	for (cur = list; cur; cur = cur->next) {
-		VCalEvent *event = (VCalEvent *)cur->data;
-		BusySpot *busy = g_new0(BusySpot, 1);
-		BusySpot *merged;
-		busy->start = icaltime_as_timet((icaltime_from_string(event->dtstart)));
-		busy->end = icaltime_as_timet((icaltime_from_string(event->dtend)));
-		busy_spots = vcal_g_slist_merge(busy_spots, busy, TRUE);
-		g_free(busy);
-		vcal_manager_free_event(event);
-	}
-
-	busy_spots = vcal_g_slist_merge_all(busy_spots);
-	debug_print("merged to %d events\n", g_slist_length(busy_spots));
-#endif
-
 	itt_start = icaltime_from_timet(whole_start, FALSE);
 	itt_end = icaltime_from_timet(whole_end, FALSE);
 	itt_start.second = itt_start.minute = itt_start.hour = 0;
@@ -2203,27 +2104,6 @@ gboolean vcal_meeting_export_freebusy(const gchar *path, const gchar *user,
 		icaltime_as_ical_string(itt_start),
 		icaltime_as_ical_string(itt_end));
 
-#if 0
-	for (cur = busy_spots; cur; cur = cur->next) {
-		icalproperty *prop;
-		struct icalperiodtype ipt;
-
-		BusySpot *spot = (BusySpot *)cur->data;
-		debug_print("FREEBUSY:%s/%s\n",
-			icaltime_as_ical_string(icaltime_from_timet(spot->start, FALSE)),
-			icaltime_as_ical_string(icaltime_from_timet(spot->end, FALSE)));
-		
-		ipt.start = icaltime_from_timet(spot->start, FALSE);
-		ipt.end = icaltime_from_timet(spot->end, FALSE);
-		ipt.duration = icaltime_subtract(ipt.end, ipt.start);
-		prop = icalproperty_new_freebusy(ipt);
-		icalcomponent_add_property(vfreebusy, prop);
-
-		g_free(spot);
-	}
-	g_slist_free(busy_spots);
-#endif
-
 	for (cur = list; cur; cur = cur->next) {
 		VCalEvent *event = (VCalEvent *)cur->data;
 		icalproperty *prop;
@@ -2262,7 +2142,6 @@ gboolean vcal_meeting_export_freebusy(const gchar *path, const gchar *user,
 	icalcomponent_free(calendar);
 	g_slist_free(list);
 	
-putfile:
 	if ((!path || strlen(path) == 0 || !vcalprefs.export_freebusy_enable)) {
 		g_free(tmpfile);
 		return TRUE;
